#include "cuz_driver.h"
#include "cuz_opengl.h"
#include "cuz_util.h"

#include <math.h>

#include <sys/types.h>
#include <sys/stat.h>
#ifndef PLATFORM_WIN32
#include <unistd.h>
#endif

#ifdef PLATFORM_WIN32
    #include <windows.h>
    #include <conio.h>
    #include <shellapi.h>
	#include <direct.h>
#endif

#ifdef PLATFORM_IOS
#include "cuz_driver-ios.h"
#endif

#ifdef USING_SDL
#include "cuz_driver-sdl.h"
#endif

//#define BUILD_DEBUG

#ifdef PLATFORM_WIN32
	extern "C" { FILE _iob[3] = {__iob_func()[0], __iob_func()[1], __iob_func()[2]}; }
	#pragma comment(lib, "SDL.lib")
	#ifdef BUILD_DEBUG
		#pragma comment(lib, "SDLmain.lib")
	#else
		#pragma comment(lib, "../../cuzcode/ext/sdl/windows/SDLmain_nostdio.lib")
	#endif
	#pragma comment(lib, "SDL_image.lib")
	#pragma comment(lib, "OpenGL32.lib")
	#pragma comment(lib, "GLu32.lib")
	#pragma comment(lib, "ws2_32.lib")
	#pragma comment(lib, "winmm.lib")

#endif


struct _DriverData {
    int width,height;
    float aspect;
    
    const char * gamename;
    char caption[256];
    int fps;
    _DriverData() {
        memset(this,0,sizeof(_DriverData));
        width=height=0;
        aspect=1;
        gamename = 0;
        fps = 0;
    }
};

static _DriverData *_driver = 0;

void driver_set_caption(const char *v) {
    sprintf(_driver->caption,"%s",v);
}
const char *driver_get_caption() {
    return _driver->caption;
}

void _driver_audio_deinit() ;
void _driver_audio_init() ;
void _driver_video_deinit() ;
void _driver_video_init() ;

void driver_init() {
    _driver = new _DriverData();

    _driver_video_init();
    _driver_audio_init();
}

const char *_driver_get_datadir() {
    static char dname[512];
    memset(dname,0,sizeof(dname));
    sprintf(dname,"../data");
    if (strlen(_driver->gamename)) {
        #if defined(PLATFORM_OSX)
        sprintf(dname,"%s.app/Contents/Resources",_driver->gamename);
        #elif defined(PLATFORM_LINUX)
        sprintf(dname,"../data");
        #elif defined(PLATFORM_WIN32)
        sprintf(dname,"./data");
        #elif defined(PLATFORM_IOS)
        sprintf(dname,"%s/%s.app",getenv("HOME"),_driver->gamename);
        #else
        #error "no PLATFORM_ defined"
        #endif
    }
    return dname;
}


void driver_get_size(int *w, int *h) {
    *w = _driver->width;
    *h = _driver->height;
}

void driver_set_size(int w, int h) {
    _driver->width = w;
    _driver->height = h;
}

float driver_get_aspect() {
    return _driver->aspect;
}
void driver_set_aspect(float v) {
    _driver->aspect = v;
}

void driver_deinit() {
    _driver_audio_deinit();
    _driver_video_deinit();
    delete _driver;
    _driver = 0;
}


int _driver_fsize(const char *fname) {
    struct stat stbuf;
	if (stat(fname, &stbuf)) { return 0; }
    int fsize = stbuf.st_size;
    return fsize;
}

const char *driver_data_fname(const char *_fname) {
    static char fname[512];
    sprintf(fname,"%s/%s",_driver_get_datadir(),_fname);
    return fname;
}
std::string driver_data_load(const char *_fname) {
    const char *fname = driver_data_fname(_fname);
    
    int fsize = _driver_fsize(fname);

    FILE* f = fopen(fname,"rb");
    char* mem = new char[fsize];
	if (f) {
		fread(mem, fsize, 1, f);
		fclose(f);
	}
    
    std::string r = std::string(mem,fsize);
    delete [] mem;
    return r;
}

int driver_tex_load(const char *_fname) {
    const char *fname = driver_data_fname(_fname);
    fprintf(stderr,"driver_tex_load %s\n",fname);   
#if defined(USING_SDL)
    return _driver_tex_load_sdl(fname);
#elif defined(PLATFORM_IOS)
    return _driver_tex_load_ios(fname);
#else
    #error "driver_tex_load undefined"
#endif
}


void driver_set_gamename(const char *s) {
    _driver->gamename = s;
}

const char *_driver_get_docdir() {
    static char dname[512];
    memset(dname,0,sizeof(dname));
    sprintf(dname,".");
    if (strlen(_driver->gamename)) {
        #if defined(PLATFORM_OSX)
        sprintf(dname,"%s/Library/Application Support/%s",getenv("HOME"),_driver->gamename);
        mkdir(dname,0755);
        #elif defined(PLATFORM_LINUX)
        sprintf(dname,"%s/.%s",getenv("HOME"),_driver->gamename);
        mkdir(dname,0755);
        #elif defined(PLATFORM_WIN32)
        sprintf(dname,"%s/%s",getenv("APPDATA"),_driver->gamename);
    //     SHGetFolderPath (NULL, CSIDL_LOCAL_APPDATA, NULL, 0, strPath);
        _mkdir(dname);
        #elif defined(PLATFORM_IOS)
        sprintf(dname,"%s",_driver_get_docdir_ios());
        #else
        #error "no PLATFORM_ defined"
        #endif
    }
    return dname;
}


const char *driver_doc_fname(const char *_fname) {
    static char fname[512]; sprintf(fname,"%s/%s",_driver_get_docdir(),_fname);
    return fname;
}


void driver_doc_save(const char *_fname, void *data, int len) {
    const char *fname = driver_doc_fname(_fname);
    FILE *f = fopen(fname,"wb");
    fwrite(data,1,len,f);
    fclose(f);
} 

//std::string driver_doc_load(const char *_fname) {
//    const char *fname = driver_doc_fname(_fname);
//    int fsize = _driver_fsize(fname);
//    FILE *f = fopen(fname,"rb");
//    if (!f) {
//        fprintf(stderr,"file not found: %s\n",fname);
//        return std::string("");
//    }
//    char *mem = new char[fsize];
//    fread(mem, fsize, 1, f);
//    fclose(f);
//    std::string r = std::string(mem,fsize);
//    delete [] mem;
//    return r;
//}

int driver_doc_load(const char *_fname, void *data, int len) {
    const char *fname = driver_doc_fname(_fname);
    int fsize = _driver_fsize(fname);
    if (fsize > len) {
        fprintf(stderr,"%s is too large\n",_fname);
        return 0;
    }
    FILE *f = fopen(fname,"rb");
    if (!f) {
        fprintf(stderr,"file not found: %s\n",fname);
        return 0;
    }
    fread(data, fsize, 1, f);
    fclose(f);
    return fsize;
}


void driver_timer_set(int fps) {
    _driver->fps = fps;
}


void driver_timer_tick() {
    if (!_driver->fps) { return; }
    
    static int init=0;
    static long frame=0;
    static long next;
    if (!init) {
        init = 1;
        next = driver_timer_get_ticks();
    }
    long cur = driver_timer_get_ticks();
    
    driver_timer_delay((int)mymax(0,next-cur));
    next += myinc(frame++,1000.0/_driver->fps);
    int tick_cnt = 1;
    while (next < cur) {
        next += myinc(frame++,1000.0/_driver->fps);
        tick_cnt ++;
        if (tick_cnt >= 8) {
            next = cur + myinc(frame++,1000.0/_driver->fps);
            break;
        }
    }
}

void driver_open_url(const char *url) {
        fprintf(stdout,"Driver::weblink: %s\n",url);
    int popup = 0;
        
        if (!popup) {
            
#if defined(USING_STEAM)
        _driver_open_url_steam(url);
            
            return;
#endif
            
        }
        
#if defined(PLATFORM_OSX)
        char s[1024];
        sprintf(s,"open \"%s\"",url);
        system(s);
#elif defined(PLATFORM_WIN32)
        fprintf(stdout,"Win32:WebPage\n");
	    ShellExecute(NULL,LPCTSTR("open"),LPCTSTR(url),NULL,NULL,SW_SHOWNORMAL);
#elif defined(PLATFORM_LINUX)
        char s[1024];
        sprintf(s,"firefox \"%s\" &",url);
        system(s);
#elif defined(PLATFORM_IOS)
    _driver_open_url_ios(url);
#else
        
        fprintf(stderr,"no browser to open?\n");
#endif
        
        return;
}


////////////////////////////////////////////////////////////////////////////////
/*
 **  ClanLib SDK
 **  Copyright (c) 1997-2005 The ClanLib Team
 **
 **  This software is provided 'as-is', without any express or implied
 **  warranty.  In no event will the authors be held liable for any damages
 **  arising from the use of this software.
 **
 **  Permission is granted to anyone to use this software for any purpose,
 **  including commercial applications, and to alter it and redistribute it
 **  freely, subject to the following restrictions:
 **
 **  1. The origin of this software must not be misrepresented; you must not
 **     claim that you wrote the original software. If you use this software
 **     in a product, an acknowledgment in the product documentation would be
 **     appreciated but is not required.
 **  2. Altered source versions must be plainly marked as such, and must not be
 **     misrepresented as being the original software.
 **  3. This notice may not be removed or altered from any source distribution.
 **
 **  Note: Some of the libraries ClanLib may link to may have additional
 **  requirements or restrictions.
 **
 **  File Author(s):
 **
 **    Seth A. Robinson
 **    (if your name is missing here, please add it)
 */


#ifdef PLATFORM_OSX
#include <Carbon/Carbon.h>
#endif

std::string driver_clipboard_get_text()
{
    
	std::string text;
    
#ifdef PLATFORM_WIN32
    
	if (OpenClipboard(NULL))
	{
        
		HANDLE hData = GetClipboardData(CF_TEXT);  //note, if we every support unicode we would use CF_UNICODE
		if (hData)
		{
			text = (char*)GlobalLock(hData);
			GlobalUnlock(hData);
		}
        
		CloseClipboard();
	}
    
#elif defined PLATFORM_OSX
    
	ScrapRef scrap = 0;
	Size len = 0;
    
	if (GetCurrentScrap(&scrap) == noErr && GetScrapFlavorSize(scrap, kScrapFlavorTypeText, &len) == noErr)
	{
        char *pBuffer = new char[len+1];
        pBuffer[len] = 0; //just in case they don't have their own null
        GetScrapFlavorData( scrap, kScrapFlavorTypeText, &len, pBuffer);
        
        //note, we might want to convert /r's to /n's here or something later
        
        text = pBuffer;
	}
    
#endif
	
	return text; //not handled for this platform, add it please!
}

void driver_clipboard_set_text(const std::string &text)
{
#ifdef PLATFORM_WIN32
    
	if (OpenClipboard(NULL))
	{
		EmptyClipboard();
        
		char * buffer;
		HGLOBAL clipbuffer;
	    clipbuffer = GlobalAlloc(GMEM_DDESHARE, text.size()+1);
		buffer = (char*)GlobalLock(clipbuffer);
		strcpy(buffer, text.c_str());
		GlobalUnlock(clipbuffer);
		SetClipboardData(CF_TEXT,clipbuffer);
		CloseClipboard();
	}
    
#elif defined PLATFORM_OSX
    
	ScrapRef myScrap = 0;
	ClearCurrentScrap(); //remove what is there
	GetCurrentScrap(&myScrap);
	PutScrapFlavor( myScrap, kScrapFlavorTypeText, 0, text.size(), text.c_str() );
#endif
    
}

//iOS
//
//string GetClipboardText()
//{
//	string text;
//	
//	UIPasteboard *pasteboard = [UIPasteboard generalPasteboard];
//    
//	if ([pasteboard containsPasteboardTypes: [NSArray arrayWithObject:@"public.utf8-plain-text"]])
//	{
//		text = [pasteboard.string cStringUsingEncoding:NSUTF8StringEncoding];
//	}
//	return text;
//}
//

//philhassey: btw, using android's clipboard is pretty darn simple I just found out:  http://thedevelopersinfo.wordpress.com/2009/10/11/how-to-use-clipboard-on-android/
//[2:00am] <SethR>
//of course it's wrong.  the correct way is:  tring data = clipboard.getText().toString();
//[2:00am] <SethR>
//tring=String

