#ifndef CUZ_WEB_H
#define CUZ_WEB_H

#ifdef __cplusplus
extern "C"
{
#endif

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <fcntl.h>

#ifdef PLATFORM_WIN32
    #include <Winsock2.h>
    #define EINPROGRESS WSAEINPROGRESS
#else
    #include <unistd.h>
    #include <netdb.h>
    #include <netinet/in.h>
    #include <sys/socket.h>
    #include <sys/poll.h>
#endif

#include <time.h>

#define GC_WEB_INIT 0
#define GC_WEB_CONNECT 1
#define GC_WEB_SEND 2
#define GC_WEB_RECEIVE 3
#define GC_WEB_CLOSED 4

typedef struct cuz_web_buffer {
    int len; int alloc;
    int pos;
    unsigned char *data;
} cuz_web_buffer;

typedef struct cuz_web {
    // public stuff
    // data,len will contain a pointer to the web response after the headers
    unsigned char *data; int len;
    // error is true if there is an error
    int error;
    
    // private stuff
    char host[256]; int port; char path[1024];
    
    cuz_web_buffer _request;
    cuz_web_buffer _response;
    int header; // length of header
    
    int state;
    long timeout, _timeout;
    int sockfd, numbytes;
//     struct hostent *he;
    struct sockaddr_in their_addr; // connector's address information 

} cuz_web;

int cuz_web_init();
void cuz_web_deinit();

cuz_web *cuz_web_create(const char *host, int port, const char *path, const void *post, int postlen, int timeout);

// returns 0 until the page request is finished / timed out / whatevers
int cuz_web_poll(cuz_web *page);

void cuz_web_destroy(cuz_web *page);
    
void cuz_web_get_header(cuz_web *page, char *dest, int dlen, const char *name);

int cuz_web_urlencode_len(const char *in, int len); 
void cuz_web_urlencode(char *out, int olen, const char *in, int ilen) ;

#ifdef __cplusplus
}
#endif

// C++ wrapper add-on
#ifdef __cplusplus

inline int web_init() { return cuz_web_init(); }
inline void web_deinit() { cuz_web_deinit(); }

#include <string>

inline std::string web_urlencode(const char *v, int len) {
    int l2 = cuz_web_urlencode_len(v,len);
    char *mem = new char[l2];
    cuz_web_urlencode(mem,l2,v,len);
    std::string tmp(mem);
    delete [] mem;
    return tmp;
}

inline std::string web_urlencode(std::string v) {
    return web_urlencode(v.c_str(),v.size());
}

struct WebRequest {
    // public editable
    std::string host;
    int port;
    std::string request;
    std::string post;
    float timeout;
    
    // private
    cuz_web *page;

    WebRequest() {
        port=80; request = "/"; timeout = 30; page = 0;
    }
    
    std::string urlencode(const char *v, int len) {
        return web_urlencode(v,len);

    }

    std::string urlencode(std::string v) {
        return web_urlencode(v);
    }
    
    void exec() {
        if (page) { return; }
        //fprintf(stderr,"exec()\n");
        page = cuz_web_create(host.c_str(),port,request.c_str(),post.c_str(),post.size(),timeout);
    }
    
    bool poll() {
        if (!page) { return false; }
        //fprintf(stderr,"x\n");
        return cuz_web_poll(page);
    }
    
    bool has_error() {
        if (!page) { return true; }
        return page->error;
    }
    
    int load_response(void *data, int len) {
        if (!page) { return 0; }
        if (len > page->len) { len = page->len; }
        memcpy(data,page->data,len);
        return len;
    }
    
    void *get_response(int *len) {
        if (!page) { *len = 0; return 0; }
        //fprintf(stderr,"get_response\n");
        *len = page->len;
        return page->data;
    }
    
    std::string get_response() {
        if (!page) { return ""; }
        std::string tmp ((char*)page->data,page->len) ; 
        return tmp;
    }
    
    std::string get_header(const char *name) {
        char r[256];
        cuz_web_get_header(page,r,256,name);
        return std::string(r);
    }
    
    void close() { 
        if (!page) { return ; }
        
        //fprintf(stderr,"close\n");
        cuz_web_destroy(page);
        page = 0;
    }
    
    WebRequest& operator= (WebRequest const& f)
    {
        if (this == &f) { return *this; }  // Gracefully handle self assignment
        
        // close our current request, before copying in the goods
        this->close();
 
        this->host = f.host;
        this->port = f.port;
        this->request = f.request;
        this->post = f.post;
        this->timeout = f.timeout;
        
        return *this;
    }    
    virtual ~WebRequest() {
        this->close();
    }
};

/* to use:

r = WebRequest();
r.host = "www.google.com"
r.port = 80
r.request = "/search?q="+r.urlencode("blah")+"&jokes="+r.urlencode("stuff324")
r.post = "data="+r.urlencode(&data,sizeof(data))
r.timeout = 60;

r.exec()

while (!r.poll()) { ... }

if (!r.has_error()) {
    std::string data = r.get_response();
    OR
    r.load_resonse(&data,sizeof(data));
}

r.close();
*/

#endif

#endif

