#include <memory.h>

#include "game.h"
#include "cuz_driver.h"
#include "cuz_gfx.h"
#include "cuz_font.h"
#include "cuz_json.h"
#include "cuz_mixer.h"
#include "cuz_web.h"
#include "cuz_opengl.h"

void hello_init(Game &g);

void game_init(Game &g) {
    hello_init(g);
}

#include <string>
#include <map>

std::map<int,State *>_game_states;
std::map<std::string,Action *>_game_actions;

void Game::register_state(int v, State *state) {
    _game_states[v] = state;
}
void register_action(const char *v, Action *action) {
    _game_actions[std::string(v)] = action;
}

void Game::settings_sync() {
    sprintf(this->settings.uid,"g:%s",this->settings.name);
    if (!strlen(this->settings.udid)) {
        sprintf(this->settings.udid,"time|%ld",(long)time(0));
    }
}

void Game::video_reset() {
    Game &game = *this;
    
    font_deinit();
    gfx_deinit();
    
    int vres = game.settings.vres;
    int ww=0,hh=0;
    if (vres>0) {
        ww = vres>>12;
        hh = vres&0xfff;
    }
    int fs = (game.settings.vfull!=-1) ^ (game._fs_toggle_tmp);
    
    ww = GAME_SW;
    hh = GAME_SH;
    fs = 0;
    
    driver_video_init(ww,hh, 640,480, 480,320, fs);

    gfx_init();
    gfx_set_size(GAME_SW,GAME_SH);
    gfx_set_orient(0);
    font_init();
}

void Game::open_url(const char *s) {
    Game &game = *this;
    driver_open_url(s);
    if (!driver_video_minimize()) {
        int fs = (game.settings.vfull!=-1) ^ (game._fs_toggle_tmp);
        if (fs) {
            game._fs_toggle_tmp ^= 1;
            game.video_reset();
            _game_states[this->state]->init();
            driver_video_minimize();
        }
    }
}

void Game::init() {
    Game &game = *this;
    
    game_init(game);

    driver_init();
    driver_set_gamename(GAME_NAME);
    driver_set_caption(GAME_TITLE);

    memset(this,0,sizeof(*this));
    int res = driver_doc_load(GAME_STATE_FNAME,this,sizeof(*this));
    if (this->version != GAME_STATE_VERSION) {
        memset(this,0,sizeof(*this));
    }
    memset(&this->settings,0,sizeof(this->settings));
    driver_doc_load(GAME_SETTINGS_FNAME,&this->settings,sizeof(this->settings));
    if (this->settings.version < GAME_SETTINGS_VERSION) {
        this->settings.music = 50;
        this->settings.sfx = 50;
    }
    if (!this->settings.vres) { this->settings.vres = -1; }
    if (!this->settings.vfull) { this->settings.vfull = 1; }
    this->settings.version = GAME_SETTINGS_VERSION;
    this->settings_sync();
    this->version = GAME_STATE_VERSION;
    this->quit = 0;
    
    driver_timer_set(GAME_FPS);
	web_init();
    this->video_reset();
    mixer_init();
    
    if (this->state == STATE_NONE) {
        this->state = STATE_TEST;
    }
    
    this->next(this->state);
    
    // some handy stats!
    fprintf(stderr,"sizeof(level) = %dk\n",sizeof(game.level)/1024);
    fprintf(stderr,"sizeof(game) = %dk\n",sizeof(game)/1024);
    
    this->play_music("xtheme");
}
    
void Game::paint() {
    gfx_clear();
    _game_states[this->state]->paint();
    driver_video_flip();
    driver_timer_tick();
}

#ifdef USING_SDL
#include "SDL.h"
#endif

void Game::event(Event e) {
    if (e.type == EVT_ORIENT) {
        gfx_set_orient(e.v);
    }
#ifdef USING_SDL
    if (e.type == EVT_SDLEVENT && e.stype == SDL_ACTIVEEVENT) {
        if ((e.b&SDL_APPACTIVE) != 0 && e.v != 0) {
            if (this->_fs_toggle_tmp) {
                this->_fs_toggle_tmp ^= 1;
                this->video_reset();
                _game_states[this->state]->init();
            }
        }
    }
#endif
    gfx_event_correct(&e);
    _game_states[this->state]->event(e);
}



void Game::loop() {
    Game &game = *this;
    _game_states[this->state]->loop();
}
    
void Game::deinit() {
    _game_states[this->state]->deinit();

    driver_doc_save(GAME_SETTINGS_FNAME,&this->settings,sizeof(this->settings));
    driver_doc_save(GAME_STATE_FNAME,this,sizeof(*this));
    
    mixer_deinit();
    font_deinit();
    gfx_deinit();
	web_deinit();
    driver_deinit();
}

void Game::next(int _state) {
    if (_game_states[this->state]) {
        _game_states[this->state]->deinit();
    }
    this->state = _state;
    _game_states[this->state]->init();
}

void Game::play_sound(const char *name) {
    float v = 1.0 * this->settings.sfx / 100.0;
    mixer_sample_play(mixer_sample_load(name),v);
}

int _game_music_id = 0;
void Game::play_music(const char *name) {
    _game_music_id = mixer_stream_load(name);
    float v = 1.0 * this->settings.music / 100.0;
    mixer_stream_play(_game_music_id,v);
}

void Game::stop_music() {
    mixer_stream_stop(_game_music_id);
}

void Game::volume_reset() {
    float v = 1.0 * this->settings.music / 100.0;
    mixer_stream_volume(_game_music_id,v);
}

